@@ -1,4 +1,4 @@
-from typing import Dict
+from typing import Dict, Optional
 import logging
 import datetime as dt
 import time
@@ -10,7 +10,7 @@ from . import titles
 from . import languages
 from . import dates
 
-__version__ = "0.7.6"
+__version__ = "0.7.9"
 
 logger = logging.getLogger(__name__)
 
@@ -21,7 +21,18 @@ STAT_NAMES = ['total', 'fetch', 'url', 'pub_date', 'content', 'title', 'language
 stats = {s: 0 for s in STAT_NAMES}
 
 
-def extract(url: str, html_text: str = None) -> Dict:
+def extract(url: str, html_text: Optional[str] = None, include_other_metadata: Optional[bool] = False) -> Dict:
+    """
+    The core method of this library - returns all the useful information extracted from the HTML of the next
+    article at the supplied URL.
+
+    :param str url: A valid URL to some news article online that we want to extract info from
+    :param str html_text: (optional) Supply the HTML text you already fetched from that URL. If None, we will download
+                          the HTML for you, with some reasonable timeout defaults.
+    :param bool include_other_metadata: Pass in true to top_image, authors, and other things returned under an `other`
+                                        property in the results. Warning - this can slow down extraction by around 5x.
+                                        In addition, we haven't tested the robustness and accuracy of these at scale.
+    """
     t0 = time.time()
     # first fetch the real content (if we need to)
     t1 = time.time()
@@ -56,7 +67,7 @@ def extract(url: str, html_text: str = None) -> Dict:
 
     # content
     t1 = time.time()
-    article = content.from_html(final_url, raw_html)
+    article = content.from_html(final_url, raw_html, include_other_metadata)
     content_duration = time.time() - t1
     stats['content'] += content_duration
 
@@ -69,21 +80,21 @@ def extract(url: str, html_text: str = None) -> Dict:
 
     # language
     t1 = time.time()
-    full_langauge = languages.from_html(raw_html, article['text'])  # could be something like "pt-br"
+    full_language = languages.from_html(raw_html, article['text'])  # could be something like "pt-br"
     language_duration = time.time() - t1
     stats['language'] += language_duration
 
     total_duration = time.time() - t0
     stats['total'] += total_duration
 
-    return dict(
+    results = dict(
         original_url=url,
         url=final_url,
         normalized_url=normalized_url,
         canonical_domain=canonical_domain,
         publication_date=pub_date,
-        language=full_langauge[:2] if full_langauge else full_langauge,  # keep this as a two-letter code, like "en"
-        full_langauge=full_langauge,  # could be a full region language code, like "en-AU"
+        language=full_language[:2] if full_language else full_language,  # keep this as a two-letter code, like "en"
+        full_language=full_language,  # could be a full region language code, like "en-AU"
         text_extraction_method=article['extraction_method'],
         article_title=article_title,
         normalized_article_title=normalized_title,
@@ -92,3 +103,13 @@ def extract(url: str, html_text: str = None) -> Dict:
         is_shortened=is_shortened_url,
         version=__version__,
     )
+    if include_other_metadata:
+        # other metadata we've done less robust validation on, but might be useful
+        results['other'] = dict(
+            raw_title=article['title'],
+            raw_publish_date=article['potential_publish_date'],
+            top_image_url=article['top_image_url'],
+            authors=article['authors'],
+        )
+
+    return results
