@@ -1,5 +1,7 @@
 import logging
 from abc import ABC, abstractmethod
+import re
+import dateparser
 import newspaper
 from goose3 import Goose
 from typing import Dict
@@ -38,13 +40,15 @@ METHOD_FAILED = 'failed'  # placeholder for stats
 method_success_stats = collections.Counter()
 
 
-def from_html(url: str, html_text: str) -> Dict:
+def from_html(url: str, html_text: str, include_metadata: bool = False) -> Dict:
     """
     Try a series of extractors to pull content out of HTML. The idea is to try as hard as can to get
     good content, but fallback to at least get something useful. The writeup at this site was very helpful:
     https://adrien.barbaresi.eu/blog/evaluating-text-extraction-python.html
     :param html_text: the raw HTML to parser
     :param url: this is useful to pass in for some metadata parsing
+    :param include_metadata: true to try and extract any other metdata the underlying extractor supports, which can
+                             create a notable performance hit
     :return: a dict of with url, text, title, publish_date, top_image_url, authors, and extraction_method keys
     """
     # now try each extractor against the same HTML
@@ -52,9 +56,10 @@ def from_html(url: str, html_text: str) -> Dict:
         try:
             # logger.debug("Trying {}".format(extractor_info['method']))
             extractor = extractor_info['instance']
-            extractor.extract(url, html_text)
+            extractor.extract(url, html_text, include_metadata)
             if extractor.worked():
                 method_success_stats[extractor.content['extraction_method']] += 1
+                extractor.content['text'] = extractor.content['text'].strip()
                 return extractor.content
         except Exception as e:
             # if the extractor fails for any reason, just continue on to the next one
@@ -64,12 +69,16 @@ def from_html(url: str, html_text: str) -> Dict:
 
 
 class AbstractExtractor(ABC):
+    """
+    Wrapper around individual libraries that can be useful to extract the news article "content" from an HTML page.
+    Once you've called `extract()` then the `content` property will be filled in.
+    """
 
     def __init__(self):
         self.content = None
 
     @abstractmethod
-    def extract(self, url: str, html_text: str):
+    def extract(self, url: str, html_text: str, include_metadata: bool = False):
         pass
 
     def worked(self) -> bool:
@@ -82,7 +91,7 @@ class AbstractExtractor(ABC):
 
 class Newspaper3kExtractor(AbstractExtractor):
 
-    def extract(self, url, html_text: str):
+    def extract(self, url, html_text: str, include_metadata: bool = False):
         doc = newspaper.Article(url)
         doc.download(input_html=html_text)
         doc.parse()
@@ -99,7 +108,7 @@ class Newspaper3kExtractor(AbstractExtractor):
 
 class GooseExtractor(AbstractExtractor):
 
-    def extract(self, url, html_text: str):
+    def extract(self, url, html_text: str, include_metadata: bool = False):
         g = Goose()
         g3_article = g.extract(raw_html=html_text)
         self.content = {
@@ -115,7 +124,7 @@ class GooseExtractor(AbstractExtractor):
 
 class BoilerPipe3Extractor(AbstractExtractor):
 
-    def extract(self, url: str, html_text: str):
+    def extract(self, url: str, html_text: str, include_metadata: bool = False):
         try:
             extractor = bp3_extractors.ArticleExtractor()
             bp_doc = extractor.get_doc(html_text)
@@ -133,28 +142,38 @@ class BoilerPipe3Extractor(AbstractExtractor):
             pass
 
 
+# Trafilatura outputs images in teh raw text in Markdown format
+markdown_img_path_pattern = re.compile(r"!\[[^\]]*\]\((.*?)\)")
+
+
 class TrafilaturaExtractor(AbstractExtractor):
 
-    def extract(self, url: str, html_text: str):
-        text = trafilatura.extract(html_text,
-                                   no_fallback=True,  # we have our own fallback chain
-                                   include_comments=False,  # we don't want comments as part of the content
-                                   deduplicate=True,  # be aggressive in removing duplicate content on a page
-                                   )
+    def extract(self, url: str, html_text: str, include_metadata: bool = False):
+        results = trafilatura.bare_extraction(html_text, with_metadata=include_metadata, url=url,
+                                              include_images=include_metadata)
+        image_urls = []
+        if include_metadata:
+            # pull out the images embedded in the markdown
+            for match in markdown_img_path_pattern.finditer(results['text']):
+                image_urls.append(match.group(1))
+            # remove the image links from the full text
+            text = markdown_img_path_pattern.sub("", results['text'])
+        else:
+            text = results['text']
         self.content = {
             'url': url,
             'text': text,
-            'title': None,
-            'potential_publish_date': None,
-            'top_image_url': None,
-            'authors': None,
+            'title': results['title'],
+            'potential_publish_date': dateparser.parse(results['date']),
+            'top_image_url': image_urls[0] if len(image_urls) > 0 else None,
+            'authors': results['author'].split(',') if results['author'] else None,
             'extraction_method': METHOD_TRAFILATURA,
         }
 
 
 class ReadabilityExtractor(AbstractExtractor):
 
-    def extract(self, url: str, html_text: str):
+    def extract(self, url: str, html_text: str, include_metadata: bool = False):
         try:
             doc = readability.Document(html_text)
             self.content = {
@@ -178,7 +197,7 @@ class RawHtmlExtractor(AbstractExtractor):
     def __init__(self):
         super(RawHtmlExtractor, self).__init__()
 
-    def extract(self, url: str, html_text: str):
+    def extract(self, url: str, html_text: str, include_metadata: bool = False):
         soup = BeautifulSoup(html_text, 'lxml')
         text = soup.find_all(string=True)
         output = ''
